@@ -48,10 +48,29 @@ class FileTests(testing.TestCase):
                 'storage_type': 'ENDURANCE',
                 'username': 'user',
                 'active_transactions': None,
-                'mount_addr': '127.0.0.1:/TEST'
+                'mount_addr': '127.0.0.1:/TEST',
+                'rep_partner_count': None
             }],
             json.loads(result.output))
 
+    @mock.patch('SoftLayer.FileStorageManager.list_file_volumes')
+    def test_volume_count(self, list_mock):
+        list_mock.return_value = [
+            {'serviceResource': {'datacenter': {'name': 'dal09'}}},
+            {'serviceResource': {'datacenter': {'name': 'ams01'}}},
+            {'serviceResource': {'datacenter': {'name': 'ams01'}}}
+        ]
+
+        result = self.run_command(['file', 'volume-count'])
+
+        self.assert_no_fail(result)
+        self.assertEqual(
+            {
+                'ams01': 2,
+                'dal09': 1
+            },
+            json.loads(result.output))
+
     def test_snapshot_list(self):
         result = self.run_command(['file', 'snapshot-list', '1234'])
 
@@ -122,13 +141,6 @@ class FileTests(testing.TestCase):
 
         self.assertEqual(2, result.exit_code)
 
-    def test_volume_order_performance_iops_out_of_range(self):
-        result = self.run_command(['file', 'volume-order',
-                                   '--storage-type=performance', '--size=20',
-                                   '--iops=80000', '--location=dal05'])
-
-        self.assertEqual(2, result.exit_code)
-
     def test_volume_order_performance_iops_not_multiple_of_100(self):
         result = self.run_command(['file', 'volume-order',
                                    '--storage-type=performance', '--size=20',
@@ -140,7 +152,8 @@ class FileTests(testing.TestCase):
         result = self.run_command(['file', 'volume-order',
                                    '--storage-type=performance', '--size=20',
                                    '--iops=100', '--location=dal05',
-                                   '--snapshot-size=10'])
+                                   '--snapshot-size=10',
+                                   '--service-offering=performance'])
 
         self.assertEqual(2, result.exit_code)
 
@@ -153,19 +166,22 @@ class FileTests(testing.TestCase):
                     {'description': 'Performance Storage'},
                     {'description': 'File Storage'},
                     {'description': '0.25 IOPS per GB'},
-                    {'description': '20 GB Storage Space'}]
+                    {'description': '20 GB Storage Space'},
+                    {'description': '10 GB Storage Space (Snapshot Space)'}]
             }
         }
 
         result = self.run_command(['file', 'volume-order',
                                    '--storage-type=performance', '--size=20',
-                                   '--iops=100', '--location=dal05'])
+                                   '--iops=100', '--location=dal05',
+                                   '--snapshot-size=10'])
 
         self.assert_no_fail(result)
         self.assertEqual(result.output,
                          'Order #478 placed successfully!\n'
                          ' > Performance Storage\n > File Storage\n'
-                         ' > 0.25 IOPS per GB\n > 20 GB Storage Space\n')
+                         ' > 0.25 IOPS per GB\n > 20 GB Storage Space\n'
+                         ' > 10 GB Storage Space (Snapshot Space)\n')
 
     def test_volume_order_endurance_tier_not_given(self):
         result = self.run_command(['file', 'volume-order',
@@ -277,6 +293,28 @@ class FileTests(testing.TestCase):
 
         self.assert_no_fail(result)
 
+    @mock.patch('SoftLayer.FileStorageManager.order_snapshot_space')
+    def test_snapshot_order_order_not_placed(self, order_mock):
+        order_mock.return_value = {}
+
+        result = self.run_command(['file', 'snapshot-order', '1234',
+                                   '--capacity=10', '--tier=0.25'])
+
+        self.assert_no_fail(result)
+        self.assertEqual(result.output,
+                         'Order could not be placed! Please verify '
+                         'your options and try again.\n')
+
+    @mock.patch('SoftLayer.FileStorageManager.order_snapshot_space')
+    def test_snapshot_order_performance_manager_error(self, order_mock):
+        order_mock.side_effect = ValueError('failure!')
+
+        result = self.run_command(['file', 'snapshot-order', '1234',
+                                   '--capacity=10', '--tier=0.25'])
+
+        self.assertEqual(2, result.exit_code)
+        self.assertEqual('Argument Error: failure!', result.exception.message)
+
     @mock.patch('SoftLayer.FileStorageManager.order_snapshot_space')
     def test_snapshot_order(self, order_mock):
         order_mock.return_value = {
@@ -399,6 +437,7 @@ class FileTests(testing.TestCase):
     def test_replication_locations_unsuccessful(self, locations_mock):
         locations_mock.return_value = False
         result = self.run_command(['file', 'replica-locations', '1234'])
+        self.assert_no_fail(result)
         self.assertEqual('No data centers compatible for replication.\n',
                          result.output)
 
